@@ -38,6 +38,8 @@ BEGIN {
         runclientoutput
         setlogfunc
         exerunner
+        subtextfile
+        subchars
         subbase64
         subnewlines
         subsha256base64file
@@ -62,7 +64,6 @@ use globalconfig qw(
 my $logfunc;      # optional reference to function for logging
 my @logmessages;  # array holding logged messages
 
-
 #######################################################################
 # Log an informational message
 # If a log callback function was set in setlogfunc, it is called. If not,
@@ -96,17 +97,38 @@ sub clearlogs {
     return $loglines;
 }
 
-
 #######################################################################
 
 sub includefile {
-    my ($f) = @_;
+    my ($f, $text) = @_;
     open(F, "<$f");
+    if($text) {
+        binmode F, ':crlf';
+    }
     my @a = <F>;
     close(F);
     return join("", @a);
 }
 
+sub subtextfile {
+    my ($thing) = @_;
+
+    my $count = ($$thing =~ s/%includetext ([^%]*)%[\n\r]+/includefile($1, 1)/ge);
+
+    return $count > 0;
+}
+
+sub subchars {
+    my ($thing) = @_;
+
+    $$thing =~ s/%SP/ /g;    # space
+    $$thing =~ s/%TAB/\t/g;  # horizontal tab
+    $$thing =~ s/%CR/\r/g;   # carriage return aka \r aka 0x0d
+    $$thing =~ s/%LT/</g;
+    $$thing =~ s/%GT/>/g;
+    $$thing =~ s/%AMP/&/g;
+}
+
 sub subbase64 {
     my ($thing) = @_;
 
@@ -147,12 +169,8 @@ sub subbase64 {
         $$thing =~ s/%%DAYS%%/%alternatives[$d,$d2]/;
     }
 
-    $$thing =~ s/%SP/ /g;    # space
-    $$thing =~ s/%TAB/\t/g;  # horizontal tab
-    $$thing =~ s/%CR/\r/g;   # carriage return aka \r aka 0x0d
-
     # include a file
-    $$thing =~ s/%include ([^%]*)%[\n\r]+/includefile($1)/ge;
+    $$thing =~ s/%include ([^%]*)%[\n\r]+/includefile($1, 0)/ge;
 }
 
 my $prevupdate;  # module scope so it remembers the last value
@@ -177,7 +195,7 @@ sub subnewlines {
     }
     else {
         if(($$thing =~ /^\n\z/) && $prevupdate) {
-            # if there's a blank link after a line we update, we hope it is
+            # if there is a blank link after a line we update, we hope it is
             # the empty line following headers
             $$thing =~ s/\x0a/\x0d\x0a/;
         }
